//
//  MOTTypeTextField.m
//  MOTTypeTextField
//
//  Created by 牛帮主 on 17/1/12.
//  Copyright © 2017年 牛帮主. All rights reserved.
//

#import "MOTTypeTextField.h"

@interface MOTTypeTextField () <UITextFieldDelegate>

@end

@implementation MOTTypeTextField

#pragma mark -懒加载
-(UITextField *)textField {
    if (!_textField) {
        _textField = [UITextField new];
        _textField.delegate = self;
        _textField.borderStyle = UITextBorderStyleLine;
        [_textField addTarget:self action:@selector(textFieldChange:) forControlEvents:UIControlEventEditingChanged];
        
        [self addSubview:_textField];
    }
    
    return _textField;
}

#pragma mark -系统
-(instancetype)init {
    if (self = [super init]) {
        [self defaultSetup];
    }
    
    return self;
}
-(instancetype)initWithCoder:(NSCoder *)aDecoder {
    if (self = [super initWithCoder:aDecoder]) {
        [self defaultSetup];
    }
    
    return self;
}
-(instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        [self defaultSetup];
    }
    
    return self;
}

#pragma mark -set方法
-(void)setTextfield_type:(TEXTFIELD_TYPE)textfield_type {
    _textfield_type = textfield_type;
}

#pragma mark -初始化方法
-(void)defaultSetup {
    [self addTextFieldLayoutConstraint];
}

#pragma mark -约束
-(void)addTextFieldLayoutConstraint {
    self.textField.translatesAutoresizingMaskIntoConstraints = NO;
    NSLayoutConstraint *left = [NSLayoutConstraint constraintWithItem:self.textField attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeLeft multiplier:1 constant:0];
    NSLayoutConstraint *right = [NSLayoutConstraint constraintWithItem:self.textField attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeRight multiplier:1 constant:0];
    NSLayoutConstraint *top = [NSLayoutConstraint constraintWithItem:self.textField attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeTop multiplier:1 constant:0];
    NSLayoutConstraint *bottom = [NSLayoutConstraint constraintWithItem:self.textField attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeBottom multiplier:1 constant:0];
    
    [self addConstraints:@[left, right, top, bottom]];
}

#pragma mark -事件
-(void)textFieldChange:(UITextField*) sender {
    if (self.textfield_type == TEXTFIELD_TYPE_POS_NUMBER) {
        if (self.numberLimit) {
            if ([sender.text intValue] > self.maxNumber) {
                sender.text = [sender.text substringToIndex:sender.text.length - 1];
            }
        }
    }
}

#pragma mark -textField 代理
-(BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    
    if ([string isEqualToString:@"\n"]) {
        if (self.returnKeyBlock) {
            self.returnKeyBlock(textField);
        }
    }
    
    if (range.length == 1 && string.length == 0) {
        return YES;
    }
    
    if (self.textfield_type == TEXTFIELD_TYPE_POS_NUMBER) {
        return [self validatePosNumber:string];
    }
    else if (self.textfield_type == TEXTFIELD_TYPE_ENGLISH_WORD) {
        return [self validateEnglishWord:string];
    }
    else if (self.textfield_type == TEXTFIELD_TYPE_NUMBER_LINE_WORD) {
        return [self validateNumberLineEnglishWord:string];
    }
    else if (self.textfield_type == TEXTFIELD_TYPE_OTHER) {
        if (self.regex.length > 0) {
            return [self validateOther:string];
        }
    }
    
    return YES;
}

#pragma mark -正则
//正则基础方法
-(BOOL)validateBaseFunctionWithString:(NSString*) string Regex:(NSString*) regex {
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", regex];
    return [predicate evaluateWithObject:string];
}

//判断是否是非负正整数
-(BOOL)validatePosNumber:(NSString *)number {
    NSString *numberRegex = @"^[1-9]\\d*|0$";
    return [self validateBaseFunctionWithString:number Regex:numberRegex];
}
-(BOOL)validateEnglishWord:(NSString *)text {
    NSString *regex = @"^[A-Za-z]+$";
    return [self validateBaseFunctionWithString:text Regex:regex];
}
-(BOOL)validateNumberLineEnglishWord:(NSString *)text {
    NSString *regex = @"^\\w+$";
    return [self validateBaseFunctionWithString:text Regex:regex];
}

//判断其他类型的正则，需要自己填写正则表达式
-(BOOL)validateOther:(NSString *) text {
    return [self validateBaseFunctionWithString:text Regex:self.regex];
}

#pragma mark -销毁
-(void)dealloc {
    //NSLog(@"MOTTypeTextField销毁了");
}

@end
